@@ -1,5 +1,5 @@
 #include "libB.hpp"
 
-std::string getZlibVers() {
+std::string getZlibVers(void) {
   return "STUB";
 }
